@@ -10,6 +10,7 @@ use warnings;
 use Debian::Debhelper::Dh_Lib;
 use File::Find;
 use JSON;
+use IPC::Run qw(run);
 use parent qw(Debian::Debhelper::Buildsystem);
 
 use constant MAX_BUILD_LOOP => 3;
@@ -46,7 +47,7 @@ sub DESCRIPTION {
 my $rootFilesIgnored =
 qr/^(?:c(?:o(?:ntribute|pying)|hange(?:log|s?)|akefile)|a(?:ut(?:otest\.watchr|hors?)|ppveyor\.yml)|g(?:ulpfile(?:\.babel)?|runt(?:file)?)\.js|b(?:inding\.gyp|ench.*\.js|ower\.json)|.*\.(?:m(?:arkdown|d)|pdf|txt)|r(?:ollup\.config\.js|eadme)|t(?:sconfig\.json|ests?\.js)|l(?:icen[cs]e.*|erna\.json)|(?:Docker|make)file|package-lock\.json|yarn\.lock|history|\..*)$/i;
 my $filesIgnored =
-qr/^(?:\.(?:git(?:ignore)?|npmignore|eslint.*)|.*\.(?:c(?:offee|\+\+|cp?)?|def|mk|h)|(?:binding.M|m)akefile)$/i;
+qr/^(?:\.(?:git(?:ignore)?|npmignore|eslint.*)|.*\.(?:c(?:offee|\+\+|cp?)?|def|mk|h)|(?:[jm]|binding.M)akefile)$/i;
 
 # Regexp compiled from dh_nodejs/dh_dirs.excluded
 my $dirIgnored =
@@ -59,9 +60,9 @@ my $authorizedExt = qr/\.(?:js)$/;
 
 sub open_file {
     my ( $self, $file ) = @_;
-    die unless $file;
+    error unless $file;
     my $f;
-    open( $f, $file ) or die $!;
+    open( $f, $file ) or error $!;
     my @lines = map {
         chomp;
         s/#.*$//;
@@ -79,8 +80,8 @@ sub main_package {
     return ( $main_p ? $main_p : () ) if defined $main_p;
     return '.' unless -e 'debian/nodejs/main';
     my @lines = $self->open_file('debian/nodejs/main');
-    die 'Malformed debian/nodejs/main' if @lines > 1;
-    die "Missing main directory $lines[0]" if @lines and !-d $lines[0];
+    error 'Malformed debian/nodejs/main'     if @lines > 1;
+    error "Missing main directory $lines[0]" if @lines and !-d $lines[0];
     $main_p = $lines[0] || '';
     return @lines;
 }
@@ -89,7 +90,7 @@ sub main_package {
 sub pjson {
     my ( $self, $dir ) = @_;
     unless ( -e "$dir/package.json" ) {
-        print STDERR "/!\\ $dir/package.json not found\n";
+        warning "/!\\ $dir/package.json not found\n";
         return undef;
     }
     return $json{$dir} if $json{$dir};
@@ -124,12 +125,14 @@ sub component_list {
         if ( -d $component ) {
             my $package;
             eval { $package = $self->pjson($component)->{name} };
-            print STDERR "Unable to load $component: $@" if $@;
-            next if ($@);
+            if ($@) {
+                warning "Unable to load $component: $@";
+                next;
+            }
             $packages{$component} = $package;
         }
         else {
-            print STDERR "Can't find $component directory in " . `pwd`;
+            warning "Can't find $component directory in " . `pwd`;
             next;
         }
     }
@@ -140,7 +143,7 @@ sub pattern {
     my ( $self, $line ) = @_;
     my ( $p, $pattern );
     print "Parsing expression $line\n";
-    $line =~ m/^(.*?)([\{\*].*)$/ or die "Bad line $line";
+    $line =~ m/^(.*?)([\{\*].*)$/ or error "Bad line $line";
     my $tmp = $line;
     my ( $dir, $expr ) = ( $1, $2 );
     if ($dir) {
@@ -164,7 +167,7 @@ sub pattern {
     my $pat = "^$p/" . join( '.*(?<=/)', @dirs ) . '$';
     print "Line $tmp becomes: $pat\n";
     eval { $pattern = qr/$pat/ };
-    die << "EOF" if ($@);
+    error << "EOF" if ($@);
 
 Unable to parse expression: "$tmp" (converted into "$pat")
 If it is correct, please fill a bug against pkg-js-tools.
@@ -189,7 +192,7 @@ sub build_module {
         print "Found debian/nodejs/$dir/build\n";
         print_and_doit(
             { chdir => $builddir },
-            'sh', '-e',
+            'sh', '-ex',
             (
                 ( $dir ne '.' ? '../' x ( scalar(@count) + 1 ) : '' )
                 . "debian/nodejs/$dir/build"
@@ -221,14 +224,15 @@ sub build_module {
             $success++;
         }
         else {
-            print STDERR '### Command "'
+            warning '### Command "'
               . join( ' ', @$cmd )
               . qq'" failed in $dir\n';
             $failures++;
         }
     }
     if ( $failures and not $success ) {
-        print STDERR <<EOF;
+        warning <<EOF;
+
 ############################################################################
 $failures failures, unable to build automatically.
 Install a debian/nodejs/$dir/build or add a "override_dh_auto_build:" target
@@ -243,7 +247,7 @@ sub resolve_command {
     my ( $self, $dir, $command, $count ) = @_;
     $count //= 0;
     if ( $count > MAX_BUILD_LOOP ) {
-        print STDERR "Max loop command exceed, aborting";
+        warning "Max loop command exceed, aborting";
         return ();
     }
     my @tmp = split /\s*&&\s*/, $command;
@@ -353,8 +357,7 @@ sub install_module {
         );
         if ( $self->pjson($dir)->{types} and not $self->pjson($dir)->{typings} )
         {
-            print STDERR
-              qq{# /!\ "types" field should be replaced by "typings"}
+            warning qq{# /!\ "types" field should be replaced by "typings"}
               . " in $dir/package.json\nPlease report this bug\n";
         }
         $noFilesField = 0;
@@ -369,7 +372,7 @@ sub install_module {
     $mainFile =~ s#^(?:\./)+##;
     $mainFile = "$dir/$mainFile";
     $mainFile = "$mainFile/index.js" if -d $mainFile;
-    print STDERR "MAIN: $mainFile\n" if (DEBUG);
+    warning "MAIN: $mainFile\n" if (DEBUG);
     $mainFile =~ s#//+#/#g;
     my $foundMain = 0;
 
@@ -379,7 +382,7 @@ sub install_module {
             ( $p, $pattern ) = $self->pattern($p);
         }
         unless ( -e $p ) {
-            print STDERR "### Missing $p, skipping\n";
+            warning "### Missing $p, skipping\n";
             next;
         }
         find(
@@ -444,14 +447,14 @@ sub install_module {
 
                 # Debug
                 elsif ( DEBUG and not DEBUGFILE ) {
-                    print STDERR "# FILE $File::Find::name\n";
+                    warning "# FILE $File::Find::name\n";
                     for ( my $i = 0 ; $i < @tests ; $i++ ) {
-                        print STDERR "#     Test$i: "
-                          . ( $tests[$i] || 0 ) . "\n";
+                        warning "#     Test$i: " . ( $tests[$i] || 0 ) . "\n";
                     }
                 }
                 if ( DEBUGFILE and $_ eq DEBUGFILE ) {
-                    print STDERR <<EOF;
+                    warning <<EOF;
+
 # FILE $File::Find::name
 #    D:            $d
 #    PATH:         $File::Find::dir
@@ -459,12 +462,11 @@ sub install_module {
 #    SKIP:         $skipFiles
 #    NOFILESFIELD: $noFilesField
 EOF
-                    print STDERR '# '
+                    warning '# '
                       . ( ( grep { $_ } @tests ) ? 'EXCLUDED' : 'INSTALLED' )
                       . ":\n";
                     for ( my $i = 0 ; $i < @tests ; $i++ ) {
-                        print STDERR "#     Test$i: "
-                          . ( $tests[$i] || 0 ) . "\n";
+                        warning "#     Test$i: " . ( $tests[$i] || 0 ) . "\n";
                     }
                 }
             },
@@ -473,7 +475,7 @@ EOF
     }
     unless ( $foundMain or !$self->main_package ) {
         if ( -e $mainFile ) {
-            print STDERR
+            warning
               "package.json#main file not installed, forcing it ($mainFile)\n";
             push @dest, [ "", $mainFile ];
         }
@@ -504,7 +506,7 @@ sub readFilesField {
     my @files = map {
         $_ = $self->toJs( $dir, $_ );
         s#^\.?/##;
-        print STDERR "$dir/package.json#files: $_ does not exists\n"
+        warning "$dir/package.json#files: $_ does not exists\n"
           unless $_ and -e "$dir/$_" or /[\{\*]/;
         $_ ? "$dir/$_" : ();
     } @lines;
@@ -530,22 +532,26 @@ sub cmp_ordered_list {
     if ( -e 'debian/nodejs/build_order' ) {
         @res = map {
             my $cmp = $_;
-            die "Unknown component $_" unless $components->{$cmp};
+            error "Unknown component $_" unless $components->{$cmp};
             @dirs = grep { $_ ne $cmp } @dirs;
             $cmp;
         } $self->open_file('debian/nodejs/build_order');
     }
     if ( -e 'debian/nodejs/component_links' ) {
         my %lt;
+        my %firsts;
         map {
-            die "Malformed line $. in debian/nodejs/component_links: $_"
+            error "Malformed line $. in debian/nodejs/component_links: $_"
               unless /^([\w\-\.\/]+)\s+([\w\-\.\/]+)$/;
-            $lt{$1} = $2;
+            $lt{$1}     = $2;
+            $firsts{$1} = 1;
         } $self->open_file('debian/nodejs/component_links');
         @dirs =
           sort {
                 ( $lt{$a} and $lt{$a} eq $b ) ? -1
               : ( $lt{$b} and $lt{$b} eq $a ) ? 1
+              : $firsts{$a}                   ? -1
+              : $firsts{$b}                   ? 1
               : $a cmp $b
           } @dirs;
     }
@@ -577,7 +583,8 @@ sub configure {
     return
          unless @dirs
       or -e 'debian/nodejs/' . $self->main_package . '/extlinks'
-      or -e 'debian/nodejs/' . $self->main_package . '/extcopies';
+      or -e 'debian/nodejs/' . $self->main_package . '/extcopies'
+      or -d 'debian/build_modules' && $self->main_package eq '.';
     $self->doit_in_builddir( 'mkdir', 'node_modules' )
       unless -e 'node_modules';
 
@@ -585,30 +592,42 @@ sub configure {
     foreach my $dir ( $self->main_package, @dirs ) {
         if ( -e "debian/nodejs/$dir/extlinks" ) {
             map {
-                my $src = `nodepath $_`;
-                chomp $src;
-                my $tmp = $src;
-                $tmp =~ s#^.*?nodejs/##;
-                $tmp =~ s#/.*?$## or $tmp = '';
-                $tmp = "$tmp/" if $tmp;
-                $self->dolink( $src, "$dir/node_modules/$tmp" );
+                my ( $in, $src );
+                run [ 'nodepath', $_ ], \$in, \$src;
+                if ( $? or not $src ) {
+                    print STDERR "### $_ NOT FOUND\n";
+                }
+                else {
+                    chomp $src;
+                    my $tmp = $src;
+                    $tmp =~ s#^.*?nodejs/##;
+                    $tmp =~ s#/.*?$## or $tmp = '';
+                    $tmp = "$tmp/" if $tmp;
+                    $self->dolink( $src, "$dir/node_modules/$tmp" );
+                }
             } $self->open_file("debian/nodejs/$dir/extlinks");
         }
         if ( -e "debian/nodejs/$dir/extcopies" ) {
             map {
-                my $src = `nodepath $_`;
-                chomp $src;
-                my $dst = $src;
-                $dst =~ s#.*?nodejs/##;
-                if ( $dst =~ s#/[^/]*$## ) {
-                    $self->doit_in_builddir( 'mkdir', '-p',
-                        "$dir/node_modules/$dst" );
+                my ( $in, $src );
+                run [ 'nodepath', $_ ], \$in, \$src;
+                if ( $? or not $src ) {
+                    print STDERR "### $_ NOT FOUND\n";
                 }
                 else {
-                    $dst = '';
+                    chomp $src;
+                    my $dst = $src;
+                    $dst =~ s#.*?nodejs/##;
+                    if ( $dst =~ s#/[^/]*$## ) {
+                        $self->doit_in_builddir( 'mkdir', '-p',
+                            "$dir/node_modules/$dst" );
+                    }
+                    else {
+                        $dst = '';
+                    }
+                    $self->doit_in_builddir( 'cp', '-r', $src,
+                        "$dir/node_modules/$dst" );
                 }
-                $self->doit_in_builddir( 'cp', '-r', $src,
-                    "$dir/node_modules/$dst" );
             } $self->open_file("debian/nodejs/$dir/extcopies");
         }
     }
@@ -623,11 +642,11 @@ sub configure {
     # Links between components
     if ( -e 'debian/nodejs/component_links' ) {
         my %links = map {
-            die "Malformed line $. in debian/nodejs/component_links: $_"
+            error "Malformed line $. in debian/nodejs/component_links: $_"
               unless /^([\w\-\.\/]+)\s+([\w\-\.\/]+)$/;
             my ( $src, $dest ) = ( $1, $2 );
             foreach my $s ( $src, $dest ) {
-                die "component_links: Unknown component $s"
+                error "component_links: Unknown component $s"
                   unless $components->{$s};
             }
             ( $src => $dest );
@@ -636,6 +655,25 @@ sub configure {
             $self->dolink( $_, "$links{$_}/node_modules/$components->{$_}" );
         }
     }
+
+    # Links all modules found in debian/build_modules
+    if ( -d 'debian/build_modules' ) {
+        opendir my $buildDir, 'debian/build_modules';
+        my @buildMods = grep { /^[^\.]/ } ( readdir $buildDir );
+        foreach my $mod (@buildMods) {
+            if ( $mod =~ /^\@/ ) {
+                opendir my $buildDir, "debian/build_modules/$mod";
+                my @subMods = grep { /^[^\.]/ } ( readdir $buildDir );
+                $self->dolink( "debian/build_modules/$mod/$_",
+                    "node_modules/$mod/$_" )
+                  foreach (@subMods);
+            }
+            else {
+                $self->dolink( "debian/build_modules/$mod",
+                    "node_modules/$mod" );
+            }
+        }
+    }
 }
 
 # build step
@@ -648,7 +686,7 @@ sub build {
         $res += $self->build_module($dir);
     }
     unless ($res) {
-        print STDERR "Aborting auto_build\n";
+        warning "Aborting auto_build\n";
 
         # To avoid hard transition, this step never fail
         #exit 1;
@@ -660,26 +698,57 @@ sub test {
     my $self       = shift;
     my $components = $self->component_list;
     my @dirs       = ( sort keys %$components );
+    my @testlinks;
+    if ( -d 'debian/tests/test_modules' ) {
+        opendir my $testDir, 'debian/tests/test_modules';
+        my @testMods = grep { /^[^\.]/ } ( readdir $testDir );
+        foreach my $mod (@testMods) {
+            if ( $mod =~ /^\@/ ) {
+                opendir my $testDir, "debian/tests/test_modules/$mod";
+                my @subMods = grep { /^[^\.]/ } ( readdir $testDir );
+                $self->dolink( "debian/tests/test_modules/$mod/$_",
+                    "node_modules/$mod/$_" )
+                  foreach (@subMods);
+                push @testlinks, map { "$mod/$_" } @subMods;
+            }
+            else {
+                $self->dolink( "debian/tests/test_modules/$mod",
+                    "node_modules/$mod" );
+                push @testlinks, $mod;
+            }
+        }
+    }
+    eval {
+        if ( my $p = $self->main_package ) {
+            $self->dolink( $p, 'node_modules/' . $self->pjson($p)->{name} );
+            push @testlinks, $self->pjson($p)->{name};
+        }
+    };
     foreach (@dirs) {
         if ( -e "debian/nodejs/$_/test" ) {
             my @count = (m#(/)#g);
             print_and_doit( { chdir => $self->get_buildpath . "/$_" },
-                'sh', '-e',
+                'sh', '-ex',
                 ( '../' x ( scalar(@count) + 1 ) ) . "debian/nodejs/$_/test" );
         }
     }
-    my $testfile =
-      ( -e 'debian/nodejs/test'
+    my $testfile = (
+        -e 'debian/nodejs/test'
         ? 'debian/nodejs/test'
-        : 'debian/tests/pkg-js/test' );
+        : 'debian/tests/pkg-js/test'
+    );
     if ( -e $testfile ) {
-        $self->doit_in_builddir( '/bin/sh', '-e', $testfile );
+        $self->doit_in_builddir( '/bin/sh', '-ex', $testfile );
     }
     else {
         $self->doit_in_builddir( '/usr/bin/node', '-e',
             'require("./' . $self->main_package . '")' )
           if $self->main_package;
     }
+    foreach (@testlinks) {
+        print "Removing node_modules/$_\n";
+        unlink "node_modules/$_";
+    }
 }
 
 # install step:
@@ -707,7 +776,7 @@ sub install {
       : '';
 
     unless ($node_module) {
-        print STDERR <<EOT;
+        warning <<EOT;
 ##############################################################
 # /!\ No name found for main module, this may break install  #
 # To fix this, fix main component path in debian/nodejs/main #
@@ -766,7 +835,7 @@ EOT
                     next CMP;
                 }
             }
-            print STDERR
+            warning
               qq{# /!\ In debian/nodejs/submodules, "$cmp" matches nothing\n};
         }
         @dirs = @tmp;
@@ -798,6 +867,33 @@ EOT
     addsubstvar( $package, 'nodejs:Provides', join( ', ', @provides ) )
       if @provides;
 
+    {
+        my ( $in, $src );
+        if ( -e 'debian/node-node-expat.substvars' ) {
+            run [
+                'grep', '-Po',
+                'shlibs:Depends=.*libnode\d+ \(>= \K\S+(?=\))',
+                'debian/node-node-expat.substvars'
+              ],
+              \$in, \$src;
+        }
+        if ( not $src ) {
+            run [
+                'dpkg-query', '--showformat=${source:Upstream-Version}',
+                '--show',     'nodejs'
+              ],
+              \$in, \$src;
+        }
+        if ( not $src or $? ) {
+            print STDERR "### node version not found\n";
+        }
+        else {
+            chomp $src;
+            addsubstvar( $package, 'nodejs:Version', $src );
+            print "Set \${nodejs:Version} to $src\n";
+        }
+    }
+
     # Links
     if ( -e "debian/nodejs/links" ) {
         my $tmp = tmpdir($package);
@@ -806,7 +902,7 @@ EOT
             my ( $src, $dest ) =
               map { m#^/# ? $_ : "$realArchPath/$_" } split /\s+/, $line;
             unless ( $src and $dest ) {
-                die "Bad line in debian/nodejs/links: $_";
+                error "Bad line in debian/nodejs/links: $_";
             }
             print "Linking $dest to $src\n";
             make_symlink( $dest, $src, $tmp );
@@ -834,7 +930,7 @@ sub clean {
     }
     if ( -e 'debian/nodejs/component_links' ) {
         map {
-            die "Malformed line $. in debian/nodejs/component_links: $_"
+            error "Malformed line $. in debian/nodejs/component_links: $_"
               unless /^([\w\-\.\/]+)\s+([\w\-\.\/]+)$/;
             my ( $src, $dest ) = ( $1, $2 );
             $self->do_unlink("$dest/node_modules/$components->{$src}");
@@ -846,6 +942,21 @@ sub clean {
         # Error not catched here
         $self->do_unlink("node_modules/$package");
     }
+    if ( -d 'debian/build_modules' ) {
+        opendir my $buildDir, 'debian/build_modules';
+        my @buildMods = grep { /^[^\.]/ } ( readdir $buildDir );
+        foreach my $mod (@buildMods) {
+            if ( $mod =~ /^\@/ ) {
+                opendir my $buildDir, "debian/build_modules/$mod";
+                my @subMods = grep { /^[^\.]/ } ( readdir $buildDir );
+                $self->do_unlink("node_modules/$mod/$_") foreach (@subMods);
+                eval { rmdir "node_modules/$mod" };
+            }
+            else {
+                $self->do_unlink("node_modules/$mod");
+            }
+        }
+    }
 }
 
 1;
