@@ -52,6 +52,7 @@ sub url_part {
     return unless ($url);
     my $ourl = $url;
     return unless $url;
+    $url =~ s#git\@(.*?):#https://$1/#;
     $url =~ s#^git.*\@#https://#;
     $url =~ s#.*git.*://#https://#;
     $url =~ s/\.git$//;
@@ -88,10 +89,11 @@ sub git_last_version {
 }
 
 sub git_watch {
-    my ( $repo, $component, $type, $version, $after ) = @_;
+    my ( $repo, $component, $type, $version, $after, $noctype ) = @_;
     $version   = $version   ? "v?($version(?:\.[\\d\\.]+)?)" : "v?([\\d\\.]+)";
     $component = $component ? "component=$component,\\\n"    : '',
       $type    = $type      ? " $type"                       : '';
+    $component .= "ctype=nodejs,\\\n" unless ($noctype);
     my $name = $repo;
     $name =~ s#.*/([^/]+)/?#$1#;
     $name ||= "to-be-fixed";
@@ -110,10 +112,11 @@ EOF
 }
 
 sub registry_watch {
-    my ( $pname, $component, $type, $version ) = @_;
+    my ( $pname, $component, $type, $version, $noctype ) = @_;
     $version   = $version   ? "($version(?:\.[\\d\\.]+)?)" : "([\\d\\.]+)";
     $component = $component ? ",component=$component"      : '';
     $type      = $type      ? " $type"                     : '';
+    $component .= ",ctype=nodejs" unless ($noctype);
     my $fname = $pname;
     $fname =~ s#.*/##;
     return <<"EOF";
